// Copyright 2024 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package jobqueue

import (
	"github.com/stretchr/testify/assert"
	"gorm.io/driver/sqlite"
	"gorm.io/gorm"
	"gorm.io/gorm/logger"
	"log"
	"os"
	"testing"
	"time"
)

func TestUpdateJob(t *testing.T) {
	db, err := gorm.Open(sqlite.Open(":memory:"), &gorm.Config{})
	assert.NoError(t, err)

	assert.NoError(t, db.AutoMigrate(&JobPersistence{}, &JobLog{}, &JobStats{}))

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job1", runner)
	assert.NoError(t, saveJob(job, db))

	job.SetPriority(PriorityHigh)
	assert.NoError(t, updateJob(job, db))

	var jobPersistence JobPersistence
	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
	assert.Equal(t, PriorityDefault, jobPersistence.Priority) // PriorityDefault because the update should not update the priority

	assert.NoError(t, saveJob(job, db))

	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
	assert.Equal(t, PriorityHigh, jobPersistence.Priority)

	var count int64
	db.Model(&JobPersistence{}).Count(&count)
	assert.Equal(t, int64(1), count) // Nur ein Datensatz sollte vorhanden sein
}

func TestCheckAndSaveOrUpdate(t *testing.T) {

	newLogger := logger.New(
		log.New(os.Stdout, "\r\n", log.LstdFlags),
		logger.Config{
			SlowThreshold: time.Second,
			LogLevel:      logger.Info,
			Colorful:      true,
		},
	)

	db, err := gorm.Open(sqlite.Open(":memory:"), &gorm.Config{
		Logger: newLogger,
	})
	assert.NoError(t, err)

	assert.NoError(t, db.AutoMigrate(&JobPersistence{}, &JobLog{}, &JobStats{}))

	runner := &CounterRunnable{}
	job := NewJob[CounterResult]("job1", runner)

	syncer := &JobSyncer{manager: &Manager{database: db}}

	var jobPersistence JobPersistence

	assert.NoError(t, syncer.CheckAndSaveOrUpdate(job))
	jobPersistence = JobPersistence{}
	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
	assert.Equal(t, PriorityDefault, jobPersistence.Priority)

	job.SetPriority(PriorityHigh)
	assert.NoError(t, syncer.CheckAndSaveOrUpdate(job))
	jobPersistence = JobPersistence{}
	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
	assert.Equal(t, PriorityDefault, jobPersistence.Priority) // second update should not update the priority, because update only update stats and logs

	// set stats and logs and test
	job.stats = JobStats{
		JobID:        job.GetID(),
		RunCount:     2,
		SuccessCount: 3,
		ErrorCount:   4,
		TimeMetrics:  TimeMetrics{},
		CreatedAt:    time.Time{},
		UpdatedAt:    time.Time{},
		DeletedAt:    gorm.DeletedAt{},
	}

	job.logs = []JobLog{
		{
			JobID: job.GetID(),
		}}

	assert.NoError(t, syncer.CheckAndSaveOrUpdate(job))

	job.stats.RunCount = 9
	assert.NoError(t, syncer.CheckAndSaveOrUpdate(job))

	jobPersistence = JobPersistence{}
	assert.NoError(t, db.First(&jobPersistence, "id = ?", job.GetID()).Error)
}
